import { isFile } from "@metamask/snaps-utils/node";
import { assert } from "@metamask/utils";
import { build } from "./implementation.mjs";
import { getBundleAnalyzerPort } from "./utils.mjs";
import { CommandError } from "../../errors.mjs";
import { success, executeSteps } from "../../utils/index.mjs";
export const steps = [
    {
        name: 'Checking the input file.',
        condition: ({ build: enableBuild }) => enableBuild,
        task: async ({ config }) => {
            const { input } = config;
            if (!(await isFile(input))) {
                throw new CommandError(`Input file not found: "${input}". Make sure that the "input" field in your Snap config is correct.`);
            }
        },
    },
    {
        name: 'Building the Snap bundle.',
        condition: ({ build: enableBuild }) => enableBuild,
        task: async (context) => {
            const { analyze, config, spinner } = context;
            const compiler = await build(config, {
                analyze,
                evaluate: config.evaluate,
                spinner,
            });
            if (analyze) {
                return {
                    ...context,
                    port: await getBundleAnalyzerPort(compiler),
                };
            }
            return undefined;
        },
    },
    {
        name: 'Running analyser.',
        condition: ({ analyze }) => analyze,
        task: async ({ spinner, port }) => {
            assert(port, 'Port is not defined.');
            success(`Bundle analyzer running at http://localhost:${port}.`, spinner);
            spinner.stop();
        },
    },
];
/**
 * Build all files in the given source directory to the given destination
 * directory.
 *
 * This creates the destination directory if it doesn't exist.
 *
 * @param config - The config object.
 * @param analyze - Whether to analyze the bundle.
 * @returns Nothing.
 */
export async function buildHandler(config, analyze = false) {
    return await executeSteps(steps, {
        build: true,
        config,
        analyze,
    });
}
//# sourceMappingURL=build.mjs.map