import type { Infer } from "@metamask/superstruct";
import type { Configuration as WebpackConfiguration } from "webpack";
import type { YargsArgs } from "./types/yargs.d.mjs";
/**
 * The configuration for the Snaps CLI, stored as `snap.config.js` or
 * `snap.config.ts` in the root of the project.
 */
export type SnapConfig = {
    /**
     * The path to the snap entry point. This should be a JavaScript or TypeScript
     * file.
     */
    input: string;
    /**
     * Whether to generate source maps for the snap. If `true`, source maps will
     * be generated as separate files. If `'inline'`, source maps will be
     * inlined in the generated JavaScript bundle.
     *
     * @default true
     */
    sourceMap?: boolean | 'inline';
    /**
     * Whether to attempt to evaluate the snap in SES. This can catch some errors
     * that would otherwise only be caught at runtime.
     *
     * @default true
     */
    evaluate?: boolean;
    output?: {
        /**
         * The path to the directory where the snap will be built. This directory
         * will be created if it doesn't exist.
         *
         * If the path is relative, it will be resolved relative to the current
         * working directory.
         *
         * @default 'dist'
         */
        path?: string;
        /**
         * The name of the JavaScript bundle file.
         *
         * @default 'bundle.js'
         */
        filename?: string;
        /**
         * Whether to clean the output directory before building the snap. If
         * `true`, the output directory will be deleted and recreated. Otherwise,
         * the output directory will be left as-is.
         *
         * @default false
         */
        clean?: boolean;
        /**
         * Whether to minimize the snap bundle. If `true`, the bundle will be
         * minified. Otherwise, the bundle will be left as-is.
         *
         * @default true
         */
        minimize?: boolean;
    };
    manifest?: {
        /**
         * The path to the snap manifest file. If the path is relative, it will be
         * resolved relative to the current working directory.
         *
         * @default 'snap.manifest.json'
         */
        path?: string;
        /**
         * Whether to automatically update the manifest. If `true`, the manifest
         * will be updated with the latest shasum of the snap bundle, and some
         * common fields will be updated if they are missing or incorrect. If
         * `false`, the manifest will be left as-is.
         *
         * @default true
         */
        update?: boolean;
    };
    server?: {
        /**
         * Whether to enable the local server. If `true`, the snap will be served
         * from a local server, when running the `watch` command. If `false`, the
         * snap will not be served.
         *
         * @default true
         */
        enabled?: boolean;
        /**
         * The root directory to serve the snap from. If the path is relative, it
         * will be resolved relative to the current working directory.
         *
         * @default `process.cwd()`
         */
        root?: string;
        /**
         * The port to run the server on.
         *
         * @default 8081
         */
        port?: number;
    };
    /**
     * The environment variables to set when building the snap. These will be
     * available in the snap as `process.env`. In addition to these environment
     * variables, the following environment variables will always be set:
     *
     * - `NODE_DEBUG`: `false`
     * - `NODE_ENV`: `'production'`
     * - `DEBUG`: `false`
     *
     * Any environment variables specified here will override these defaults. You
     * can also override any variables here by setting them in your shell when
     * running the CLI.
     */
    environment?: Record<string, unknown>;
    /**
     * Options that control the logging output of the CLI.
     */
    stats?: {
        /**
         * Whether to enable verbose logging.
         *
         * @default false
         */
        verbose?: boolean;
        /**
         * Whether to log warnings about unresolved built-in modules. If `false`,
         * warnings will not be logged.
         */
        builtIns?: {
            /**
             * The built-in modules to ignore when resolving modules. If a module
             * is ignored, no warning will be logged if it is not resolved.
             */
            ignore?: string[];
        } | false;
        /**
         * Whether to log warnings about the use of the `Buffer` global. If `false`,
         * warnings will not be logged. If `true`, the CLI will warn if the `Buffer`
         * global is used, but not provided by Webpack's `DefinePlugin`.
         */
        buffer?: boolean;
    };
    /**
     * Whether to provide polyfills for node builtins. If `true`, all the available
     * polyfills will be provided. If `false` no polyfills will be provided. If a
     * configuration object is passed only the polyfills set to `true` will be provided.
     *
     * @default false
     * @example
     * ```ts
     * polyfills: true
     *
     * // or
     *
     * polyfills: {
     *  assert: true,
     *  buffer: true
     * }
     * ```
     */
    polyfills?: boolean | {
        assert?: boolean;
        buffer?: boolean;
        console?: boolean;
        constants?: boolean;
        crypto?: boolean;
        domain?: boolean;
        events?: boolean;
        http?: boolean;
        https?: boolean;
        os?: boolean;
        path?: boolean;
        punycode?: boolean;
        process?: boolean;
        querystring?: boolean;
        stream?: boolean;
        _stream_duplex?: boolean;
        _stream_passthrough?: boolean;
        _stream_readable?: boolean;
        _stream_transform?: boolean;
        _stream_writable?: boolean;
        string_decoder?: boolean;
        sys?: boolean;
        timers?: boolean;
        tty?: boolean;
        url?: boolean;
        util?: boolean;
        vm?: boolean;
        zlib?: boolean;
    };
    /**
     * Support for TypeScript type-checking feature.
     *
     * @example
     * {
     *   enabled: true;
     *   configFile: './path/to/tsconfig.json'
     * }
     */
    typescript?: {
        /**
         * Whether to enable TypeScript type-checking feature.
         *
         * @default false
         */
        enabled?: boolean;
        /**
         * Path to tsconfig.json file for the Snap.
         *
         * @default 'tsconfig.json'
         */
        configFile?: string;
    };
    /**
     * Optional features to enable in the CLI.
     *
     * @example
     * {
     *   features: {
     *     images: true,
     *   }
     * }
     */
    features?: {
        /**
         * Whether to enable support for images. If `true`, the Webpack
         * configuration will be modified to support images. If `false`, the
         * Webpack configuration will not be modified.
         *
         * @default true
         */
        images?: boolean;
    };
    /**
     * A function to customize the Webpack configuration used to build the snap.
     * This function will be called with the default Webpack configuration, and
     * should return the modified configuration. If not specified, the default
     * configuration will be used.
     *
     * It's recommended to use the `webpack-merge` package to merge the default
     * configuration with your customizations. The merge function is exported as
     * `merge` from the `@metamask/snaps-cli` package.
     *
     * @example
     * ```ts
     * import type { SnapsConfig } from '@metamask/snaps-cli';
     * import { merge } from '@metamask/snaps-cli';
     *
     * const config: SnapsConfig = {
     *   bundler: 'webpack',
     *   entry: 'src/index.ts',
     *   customizeWebpackConfig: (config) => merge(config, {
     *     module: {
     *       rules: [
     *         {
     *           test: /\.wasm$/,
     *           type: 'assets/resource',
     *         },
     *       ],
     *     },
     *   }),
     * };
     *
     * export default config;
     * ```
     */
    customizeWebpackConfig?: (config: WebpackConfiguration) => WebpackConfiguration;
    /**
     * Experimental features that can be enabled. These features are not
     * guaranteed to be stable, and may be removed or changed in a future release.
     */
    experimental?: {
        /**
         * Whether to enable WebAssembly support. If `true`, the Webpack
         * configuration will be modified to support WebAssembly. If `false`, the
         * Webpack configuration will not be modified.
         *
         * @default false
         */
        wasm?: boolean;
    };
};
type SnapsWebpackCustomizeWebpackConfigFunction = (config: WebpackConfiguration) => WebpackConfiguration;
export declare const SnapsConfigStruct: import("@metamask/superstruct").Struct<{
    input: string;
    sourceMap: boolean | "inline";
    evaluate: boolean;
    output: {
        path: string;
        filename: string;
        clean: boolean;
        minimize: boolean;
    };
    manifest: {
        path: string;
        update: boolean;
    };
    server: {
        port: number;
        root: string;
        enabled: boolean;
    };
    environment: Record<string, unknown>;
    stats: {
        verbose: boolean;
        builtIns: false | {
            ignore: string[];
        };
        buffer: boolean;
    };
    polyfills: boolean | {
        stream: boolean;
        path: boolean;
        buffer: boolean;
        assert: boolean;
        console: boolean;
        constants: boolean;
        crypto: boolean;
        domain: boolean;
        events: boolean;
        http: boolean;
        https: boolean;
        os: boolean;
        punycode: boolean;
        process: boolean;
        querystring: boolean;
        _stream_duplex: boolean;
        _stream_passthrough: boolean;
        _stream_readable: boolean;
        _stream_transform: boolean;
        _stream_writable: boolean;
        string_decoder: boolean;
        sys: boolean;
        timers: boolean;
        tty: boolean;
        url: boolean;
        util: boolean;
        vm: boolean;
        zlib: boolean;
    };
    typescript: {
        enabled: boolean;
        configFile: string;
    };
    features: {
        images: boolean;
    };
    experimental: {
        wasm: boolean;
    };
    customizeWebpackConfig?: SnapsWebpackCustomizeWebpackConfigFunction | undefined;
}, {
    input: import("@metamask/superstruct").Struct<string, null>;
    sourceMap: import("@metamask/superstruct").Struct<boolean | "inline", [head: import("@metamask/superstruct").Struct<boolean, null>, import("@metamask/superstruct").Struct<"inline", null>]>;
    evaluate: import("@metamask/superstruct").Struct<boolean, null>;
    output: import("@metamask/superstruct").Struct<{
        path: string;
        filename: string;
        clean: boolean;
        minimize: boolean;
    }, {
        path: import("@metamask/superstruct").Struct<string, null>;
        filename: import("@metamask/superstruct").Struct<string, null>;
        clean: import("@metamask/superstruct").Struct<boolean, null>;
        minimize: import("@metamask/superstruct").Struct<boolean, null>;
    }>;
    manifest: import("@metamask/superstruct").Struct<{
        path: string;
        update: boolean;
    }, {
        path: import("@metamask/superstruct").Struct<string, null>;
        update: import("@metamask/superstruct").Struct<boolean, null>;
    }>;
    server: import("@metamask/superstruct").Struct<{
        port: number;
        root: string;
        enabled: boolean;
    }, {
        enabled: import("@metamask/superstruct").Struct<boolean, null>;
        root: import("@metamask/superstruct").Struct<string, null>;
        port: import("@metamask/superstruct").Struct<number, null>;
    }>;
    environment: import("@metamask/superstruct").Struct<Record<string, unknown>, null>;
    stats: import("@metamask/superstruct").Struct<{
        verbose: boolean;
        builtIns: false | {
            ignore: string[];
        };
        buffer: boolean;
    }, {
        verbose: import("@metamask/superstruct").Struct<boolean, null>;
        builtIns: import("@metamask/superstruct").Struct<false | {
            ignore: string[];
        }, [head: import("@metamask/superstruct").Struct<{
            ignore: string[];
        }, {
            ignore: import("@metamask/superstruct").Struct<string[], import("@metamask/superstruct").Struct<string, null>>;
        }>, import("@metamask/superstruct").Struct<false, null>]>;
        buffer: import("@metamask/superstruct").Struct<boolean, null>;
    }>;
    polyfills: import("@metamask/superstruct").Struct<boolean | {
        stream: boolean;
        path: boolean;
        buffer: boolean;
        assert: boolean;
        console: boolean;
        constants: boolean;
        crypto: boolean;
        domain: boolean;
        events: boolean;
        http: boolean;
        https: boolean;
        os: boolean;
        punycode: boolean;
        process: boolean;
        querystring: boolean;
        _stream_duplex: boolean;
        _stream_passthrough: boolean;
        _stream_readable: boolean;
        _stream_transform: boolean;
        _stream_writable: boolean;
        string_decoder: boolean;
        sys: boolean;
        timers: boolean;
        tty: boolean;
        url: boolean;
        util: boolean;
        vm: boolean;
        zlib: boolean;
    }, [head: import("@metamask/superstruct").Struct<boolean, null>, import("@metamask/superstruct").Struct<{
        stream: boolean;
        path: boolean;
        buffer: boolean;
        assert: boolean;
        console: boolean;
        constants: boolean;
        crypto: boolean;
        domain: boolean;
        events: boolean;
        http: boolean;
        https: boolean;
        os: boolean;
        punycode: boolean;
        process: boolean;
        querystring: boolean;
        _stream_duplex: boolean;
        _stream_passthrough: boolean;
        _stream_readable: boolean;
        _stream_transform: boolean;
        _stream_writable: boolean;
        string_decoder: boolean;
        sys: boolean;
        timers: boolean;
        tty: boolean;
        url: boolean;
        util: boolean;
        vm: boolean;
        zlib: boolean;
    }, {
        assert: import("@metamask/superstruct").Struct<boolean, null>;
        buffer: import("@metamask/superstruct").Struct<boolean, null>;
        console: import("@metamask/superstruct").Struct<boolean, null>;
        constants: import("@metamask/superstruct").Struct<boolean, null>;
        crypto: import("@metamask/superstruct").Struct<boolean, null>;
        domain: import("@metamask/superstruct").Struct<boolean, null>;
        events: import("@metamask/superstruct").Struct<boolean, null>;
        http: import("@metamask/superstruct").Struct<boolean, null>;
        https: import("@metamask/superstruct").Struct<boolean, null>;
        os: import("@metamask/superstruct").Struct<boolean, null>;
        path: import("@metamask/superstruct").Struct<boolean, null>;
        punycode: import("@metamask/superstruct").Struct<boolean, null>;
        process: import("@metamask/superstruct").Struct<boolean, null>;
        querystring: import("@metamask/superstruct").Struct<boolean, null>;
        stream: import("@metamask/superstruct").Struct<boolean, null>;
        _stream_duplex: import("@metamask/superstruct").Struct<boolean, null>;
        _stream_passthrough: import("@metamask/superstruct").Struct<boolean, null>;
        _stream_readable: import("@metamask/superstruct").Struct<boolean, null>;
        _stream_transform: import("@metamask/superstruct").Struct<boolean, null>;
        _stream_writable: import("@metamask/superstruct").Struct<boolean, null>;
        string_decoder: import("@metamask/superstruct").Struct<boolean, null>;
        sys: import("@metamask/superstruct").Struct<boolean, null>;
        timers: import("@metamask/superstruct").Struct<boolean, null>;
        tty: import("@metamask/superstruct").Struct<boolean, null>;
        url: import("@metamask/superstruct").Struct<boolean, null>;
        util: import("@metamask/superstruct").Struct<boolean, null>;
        vm: import("@metamask/superstruct").Struct<boolean, null>;
        zlib: import("@metamask/superstruct").Struct<boolean, null>;
    }>]>;
    typescript: import("@metamask/superstruct").Struct<{
        enabled: boolean;
        configFile: string;
    }, {
        enabled: import("@metamask/superstruct").Struct<boolean, null>;
        configFile: import("@metamask/superstruct").Struct<string, null>;
    }>;
    features: import("@metamask/superstruct").Struct<{
        images: boolean;
    }, {
        images: import("@metamask/superstruct").Struct<boolean, null>;
    }>;
    customizeWebpackConfig: import("@metamask/superstruct").Struct<SnapsWebpackCustomizeWebpackConfigFunction | undefined, null>;
    experimental: import("@metamask/superstruct").Struct<{
        wasm: boolean;
    }, {
        wasm: import("@metamask/superstruct").Struct<boolean, null>;
    }>;
}>;
export type ProcessedConfig = Infer<typeof SnapsConfigStruct>;
/**
 * Get a validated snap config. This validates the config and adds default
 * values for any missing properties.
 *
 * @param config - The config to validate.
 * @returns The validated config.
 */
export declare function getConfig(config: unknown): ProcessedConfig;
/**
 * Load a snap config from a file. This supports both JavaScript and TypeScript
 * config files, in the CommonJS module format and the ES module format.
 *
 * This assumes that the config file exports a default object, either through
 * `module.exports` or `export default`.
 *
 * @param path - The full path to the config file.
 * @returns The validated config.
 * @throws If the config file is invalid, or if the config file does not have a
 * default export.
 */
export declare function loadConfig(path: string): Promise<{
    input: string;
    sourceMap: boolean | "inline";
    evaluate: boolean;
    output: {
        path: string;
        filename: string;
        clean: boolean;
        minimize: boolean;
    };
    manifest: {
        path: string;
        update: boolean;
    };
    server: {
        port: number;
        root: string;
        enabled: boolean;
    };
    environment: Record<string, unknown>;
    stats: {
        verbose: boolean;
        builtIns: false | {
            ignore: string[];
        };
        buffer: boolean;
    };
    polyfills: boolean | {
        stream: boolean;
        path: boolean;
        buffer: boolean;
        assert: boolean;
        console: boolean;
        constants: boolean;
        crypto: boolean;
        domain: boolean;
        events: boolean;
        http: boolean;
        https: boolean;
        os: boolean;
        punycode: boolean;
        process: boolean;
        querystring: boolean;
        _stream_duplex: boolean;
        _stream_passthrough: boolean;
        _stream_readable: boolean;
        _stream_transform: boolean;
        _stream_writable: boolean;
        string_decoder: boolean;
        sys: boolean;
        timers: boolean;
        tty: boolean;
        url: boolean;
        util: boolean;
        vm: boolean;
        zlib: boolean;
    };
    typescript: {
        enabled: boolean;
        configFile: string;
    };
    features: {
        images: boolean;
    };
    experimental: {
        wasm: boolean;
    };
    customizeWebpackConfig?: SnapsWebpackCustomizeWebpackConfigFunction | undefined;
}>;
/**
 * Resolve a snap config. This function will look for a `snap.config.js` or
 * `snap.config.ts` file in the current or specified directory.
 *
 * @param path - The path to resolve the snap config from. Defaults to the
 * current working directory.
 * @returns The resolved and validated snap config.
 * @throws If a snap config could not be found.
 */
export declare function resolveConfig(path: string): Promise<{
    input: string;
    sourceMap: boolean | "inline";
    evaluate: boolean;
    output: {
        path: string;
        filename: string;
        clean: boolean;
        minimize: boolean;
    };
    manifest: {
        path: string;
        update: boolean;
    };
    server: {
        port: number;
        root: string;
        enabled: boolean;
    };
    environment: Record<string, unknown>;
    stats: {
        verbose: boolean;
        builtIns: false | {
            ignore: string[];
        };
        buffer: boolean;
    };
    polyfills: boolean | {
        stream: boolean;
        path: boolean;
        buffer: boolean;
        assert: boolean;
        console: boolean;
        constants: boolean;
        crypto: boolean;
        domain: boolean;
        events: boolean;
        http: boolean;
        https: boolean;
        os: boolean;
        punycode: boolean;
        process: boolean;
        querystring: boolean;
        _stream_duplex: boolean;
        _stream_passthrough: boolean;
        _stream_readable: boolean;
        _stream_transform: boolean;
        _stream_writable: boolean;
        string_decoder: boolean;
        sys: boolean;
        timers: boolean;
        tty: boolean;
        url: boolean;
        util: boolean;
        vm: boolean;
        zlib: boolean;
    };
    typescript: {
        enabled: boolean;
        configFile: string;
    };
    features: {
        images: boolean;
    };
    experimental: {
        wasm: boolean;
    };
    customizeWebpackConfig?: SnapsWebpackCustomizeWebpackConfigFunction | undefined;
}>;
/**
 * Get a snap config from the CLI arguments. This will either load the config
 * from the specified config file, or resolve the config from the current
 * working directory.
 *
 * @param argv - The CLI arguments.
 * @param cwd - The current working directory. Defaults to `process.cwd()`.
 * @returns The resolved and validated snap config.
 */
export declare function getConfigByArgv(argv: YargsArgs, cwd?: string): Promise<{
    input: string;
    sourceMap: boolean | "inline";
    evaluate: boolean;
    output: {
        path: string;
        filename: string;
        clean: boolean;
        minimize: boolean;
    };
    manifest: {
        path: string;
        update: boolean;
    };
    server: {
        port: number;
        root: string;
        enabled: boolean;
    };
    environment: Record<string, unknown>;
    stats: {
        verbose: boolean;
        builtIns: false | {
            ignore: string[];
        };
        buffer: boolean;
    };
    polyfills: boolean | {
        stream: boolean;
        path: boolean;
        buffer: boolean;
        assert: boolean;
        console: boolean;
        constants: boolean;
        crypto: boolean;
        domain: boolean;
        events: boolean;
        http: boolean;
        https: boolean;
        os: boolean;
        punycode: boolean;
        process: boolean;
        querystring: boolean;
        _stream_duplex: boolean;
        _stream_passthrough: boolean;
        _stream_readable: boolean;
        _stream_transform: boolean;
        _stream_writable: boolean;
        string_decoder: boolean;
        sys: boolean;
        timers: boolean;
        tty: boolean;
        url: boolean;
        util: boolean;
        vm: boolean;
        zlib: boolean;
    };
    typescript: {
        enabled: boolean;
        configFile: string;
    };
    features: {
        images: boolean;
    };
    experimental: {
        wasm: boolean;
    };
    customizeWebpackConfig?: SnapsWebpackCustomizeWebpackConfigFunction | undefined;
}>;
export {};
//# sourceMappingURL=config.d.mts.map