import { literal, union } from "@metamask/snaps-sdk";
import { createFromStruct, indent, isFile, SnapsStructError } from "@metamask/snaps-utils/node";
import { array, boolean, defaulted, define, func, number, object, optional, record, string, unknown } from "@metamask/superstruct";
import { hasProperty } from "@metamask/utils";
import { transform } from "@swc/core";
import $chalk from "chalk";
const { dim } = $chalk;
import { readFile } from "fs/promises";
import Module from "module";
import { dirname, resolve } from "path";
import { ConfigError } from "./errors.mjs";
import { file } from "./structs.mjs";
import { CONFIG_FILE, TS_CONFIG_FILE } from "./utils/index.mjs";
const CONFIG_FILES = [CONFIG_FILE, TS_CONFIG_FILE];
// This struct is essentially the same as the `func` struct, but it's defined
// separately so that we include the function type in the inferred TypeScript
// type definitions.
const SnapsWebpackCustomizeWebpackConfigFunctionStruct = define('function', func().validator);
export const SnapsConfigStruct = object({
    input: defaulted(file(), resolve(process.cwd(), 'src/index.js')),
    sourceMap: defaulted(union([boolean(), literal('inline')]), false),
    evaluate: defaulted(boolean(), true),
    output: defaulted(object({
        path: defaulted(file(), resolve(process.cwd(), 'dist')),
        filename: defaulted(string(), 'bundle.js'),
        clean: defaulted(boolean(), false),
        minimize: defaulted(boolean(), true),
    }), {}),
    manifest: defaulted(object({
        path: defaulted(file(), resolve(process.cwd(), 'snap.manifest.json')),
        update: defaulted(boolean(), true),
    }), {}),
    server: defaulted(object({
        enabled: defaulted(boolean(), true),
        root: defaulted(file(), process.cwd()),
        port: defaulted(number(), 8081),
    }), {}),
    environment: defaulted(record(string(), unknown()), {}),
    stats: defaulted(object({
        verbose: defaulted(boolean(), false),
        builtIns: defaulted(union([
            object({ ignore: defaulted(array(string()), []) }),
            literal(false),
        ]), {}),
        buffer: defaulted(boolean(), true),
    }), {}),
    polyfills: defaulted(union([
        boolean(),
        object({
            assert: defaulted(boolean(), false),
            buffer: defaulted(boolean(), false),
            console: defaulted(boolean(), false),
            constants: defaulted(boolean(), false),
            crypto: defaulted(boolean(), false),
            domain: defaulted(boolean(), false),
            events: defaulted(boolean(), false),
            http: defaulted(boolean(), false),
            https: defaulted(boolean(), false),
            os: defaulted(boolean(), false),
            path: defaulted(boolean(), false),
            punycode: defaulted(boolean(), false),
            process: defaulted(boolean(), false),
            querystring: defaulted(boolean(), false),
            stream: defaulted(boolean(), false),
            /* eslint-disable @typescript-eslint/naming-convention */
            _stream_duplex: defaulted(boolean(), false),
            _stream_passthrough: defaulted(boolean(), false),
            _stream_readable: defaulted(boolean(), false),
            _stream_transform: defaulted(boolean(), false),
            _stream_writable: defaulted(boolean(), false),
            string_decoder: defaulted(boolean(), false),
            /* eslint-enable @typescript-eslint/naming-convention */
            sys: defaulted(boolean(), false),
            timers: defaulted(boolean(), false),
            tty: defaulted(boolean(), false),
            url: defaulted(boolean(), false),
            util: defaulted(boolean(), false),
            vm: defaulted(boolean(), false),
            zlib: defaulted(boolean(), false),
        }),
    ]), false),
    typescript: defaulted(object({
        enabled: defaulted(boolean(), false),
        configFile: defaulted(file(), 'tsconfig.json'),
    }), {}),
    features: defaulted(object({
        images: defaulted(boolean(), true),
    }), {}),
    customizeWebpackConfig: optional(SnapsWebpackCustomizeWebpackConfigFunctionStruct),
    experimental: defaulted(object({
        wasm: defaulted(boolean(), false),
    }), {}),
});
/**
 * Get a validated snap config. This validates the config and adds default
 * values for any missing properties.
 *
 * @param config - The config to validate.
 * @returns The validated config.
 */
export function getConfig(config) {
    const prefix = 'The snap config file is invalid';
    const suffix = dim('Refer to the documentation for more information: https://docs.metamask.io/snaps/reference/cli/options/');
    return createFromStruct(config, SnapsConfigStruct, prefix, suffix);
}
/**
 * Load a snap config from a file. This supports both JavaScript and TypeScript
 * config files, in the CommonJS module format and the ES module format.
 *
 * This assumes that the config file exports a default object, either through
 * `module.exports` or `export default`.
 *
 * @param path - The full path to the config file.
 * @returns The validated config.
 * @throws If the config file is invalid, or if the config file does not have a
 * default export.
 */
export async function loadConfig(path) {
    try {
        const contents = await readFile(path, 'utf8');
        const source = await transform(contents, {
            swcrc: false,
            jsc: {
                parser: {
                    syntax: 'typescript',
                },
            },
            module: {
                type: 'commonjs',
            },
        });
        const config = new Module(path);
        // @ts-expect-error - This function is not typed.
        config.paths = Module._nodeModulePaths(dirname(path));
        // @ts-expect-error - This function is not typed.
        config._compile(source.code, path);
        if (!hasProperty(config.exports, 'default')) {
            return getConfig(config.exports);
        }
        return getConfig(config.exports.default);
    }
    catch (error) {
        if (error instanceof SnapsStructError) {
            throw new ConfigError(error.message);
        }
        throw new ConfigError(`Unable to load snap config file at "${path}".\n\n${indent(error.message)}`);
    }
}
/**
 * Resolve a snap config. This function will look for a `snap.config.js` or
 * `snap.config.ts` file in the current or specified directory.
 *
 * @param path - The path to resolve the snap config from. Defaults to the
 * current working directory.
 * @returns The resolved and validated snap config.
 * @throws If a snap config could not be found.
 */
export async function resolveConfig(path) {
    for (const configFile of CONFIG_FILES) {
        const filePath = resolve(path, configFile);
        if (await isFile(filePath)) {
            return await loadConfig(filePath);
        }
    }
    throw new ConfigError(`Could not find a "snap.config.js" or "snap.config.ts" file in the current or specified directory ("${path}").`);
}
/**
 * Get a snap config from the CLI arguments. This will either load the config
 * from the specified config file, or resolve the config from the current
 * working directory.
 *
 * @param argv - The CLI arguments.
 * @param cwd - The current working directory. Defaults to `process.cwd()`.
 * @returns The resolved and validated snap config.
 */
export async function getConfigByArgv(argv, cwd = process.cwd()) {
    if (argv.config) {
        if (!(await isFile(argv.config))) {
            throw new ConfigError(`Could not find a config file at "${argv.config}". Make sure that the path is correct.`);
        }
        return await loadConfig(argv.config);
    }
    return await resolveConfig(cwd);
}
//# sourceMappingURL=config.mjs.map