"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SnapsBundleWarningsPlugin = exports.SnapsBuiltInResolver = exports.SnapsWatchPlugin = exports.SnapsStatsPlugin = void 0;
const utils_1 = require("@metamask/utils");
const chalk_1 = require("chalk");
const module_1 = require("module");
const webpack_1 = require("webpack");
const utils_2 = require("./utils.cjs");
const eval_1 = require("../commands/eval/index.cjs");
const utils_3 = require("../utils/index.cjs");
/**
 * A plugin that logs the stats after compilation. This is useful for logging
 * the number of files compiled, and the time taken to compile them.
 */
class SnapsStatsPlugin {
    /**
     * The options for the plugin.
     */
    options;
    /**
     * The spinner to use for logging.
     */
    #spinner;
    constructor(options = {
        verbose: false,
    }, spinner) {
        this.options = options;
        this.#spinner = spinner;
    }
    /**
     * Apply the plugin to the Webpack compiler.
     *
     * @param compiler - The Webpack compiler.
     */
    apply(compiler) {
        compiler.hooks.afterDone.tap(this.constructor.name, (stats) => {
            if (!stats) {
                return;
            }
            const { modules, time, errors, warnings } = stats.toJson();
            (0, utils_1.assert)(modules, 'Modules must be defined in stats.');
            (0, utils_1.assert)(time, 'Time must be defined in stats.');
            if (errors?.length) {
                const formattedErrors = errors
                    .map((statsError) => this.#getStatsErrorMessage(statsError, chalk_1.red))
                    .join('\n\n');
                (0, utils_3.error)(`Compiled ${modules.length} ${(0, utils_2.pluralize)(modules.length, 'file')} in ${time}ms with ${errors.length} ${(0, utils_2.pluralize)(errors.length, 'error')}.\n\n${formattedErrors}\n`, this.#spinner);
                this.#spinner?.stop();
                process.exitCode = 1;
                return;
            }
            if (warnings?.length) {
                const formattedWarnings = warnings
                    .map((statsWarning) => this.#getStatsErrorMessage(statsWarning, chalk_1.yellow))
                    .join('\n\n');
                (0, utils_3.warn)(`Compiled ${modules.length} ${(0, utils_2.pluralize)(modules.length, 'file')} in ${time}ms with ${warnings.length} ${(0, utils_2.pluralize)(warnings.length, 'warning')}.\n\n${formattedWarnings}\n`, this.#spinner);
            }
            else {
                (0, utils_3.info)(`Compiled ${modules.length} ${(0, utils_2.pluralize)(modules.length, 'file')} in ${time}ms.`, this.#spinner);
            }
            if (compiler.watchMode) {
                // The spinner may be restarted by the watch plugin, outside of the
                // `executeSteps` flow, so we stop it here just in case.
                this.#spinner?.succeed('Done!');
            }
        });
    }
    /**
     * Get the error message for the given stats error.
     *
     * @param statsError - The stats error.
     * @param color - The color to use for the error message.
     * @returns The error message.
     */
    #getStatsErrorMessage(statsError, color) {
        const baseMessage = this.options.verbose
            ? (0, utils_3.getErrorMessage)(statsError)
            : statsError.message;
        const [first, ...rest] = baseMessage.split('\n');
        return [
            color((0, utils_2.formatText)(`• ${first}`, 4, 2)),
            ...rest.map((message) => (0, utils_2.formatText)(color(message), 4)),
            statsError.details && `\n${(0, utils_2.formatText)((0, chalk_1.dim)(statsError.details), 6)}`,
        ]
            .filter(Boolean)
            .join('\n');
    }
}
exports.SnapsStatsPlugin = SnapsStatsPlugin;
/**
 * A plugin that adds extra files to watch. This is useful for watching files
 * that are not imported by the entry point, such as the `snap.manifest.json`
 * file.
 */
class SnapsWatchPlugin {
    /**
     * The options for the plugin.
     */
    options;
    /**
     * The spinner to use for logging.
     */
    #spinner;
    constructor(options, spinner) {
        this.options = options;
        this.#spinner = spinner;
    }
    /**
     * Apply the plugin to the Webpack compiler.
     *
     * @param compiler - The Webpack compiler.
     */
    apply(compiler) {
        compiler.hooks.invalid.tap(this.constructor.name, (file) => {
            this.#spinner?.start();
            (0, utils_3.info)(`Changes detected in ${(0, chalk_1.yellow)(file)}, recompiling.`, this.#spinner);
        });
        compiler.hooks.afterEmit.tapPromise(this.constructor.name, async ({ fileDependencies }) => {
            this.options.files?.forEach(fileDependencies.add.bind(fileDependencies));
            if (this.options.bundle && this.options.evaluate) {
                await this.#safeEvaluate(this.options.bundle);
            }
        });
    }
    /**
     * Safely evaluate the bundle at the given path. If an error occurs, it will
     * be logged to the console, rather than throwing an error.
     *
     * This function should never throw an error.
     *
     * @param bundlePath - The path to the bundle.
     */
    async #safeEvaluate(bundlePath) {
        try {
            await (0, eval_1.evaluate)(bundlePath);
            (0, utils_3.info)(`Snap bundle evaluated successfully.`, this.#spinner);
        }
        catch (evaluateError) {
            (0, utils_3.error)(evaluateError.message, this.#spinner);
        }
    }
}
exports.SnapsWatchPlugin = SnapsWatchPlugin;
/**
 * A plugin that logs a message when a built-in module is not resolved. The
 * MetaMask Snaps CLI does not support built-in modules by default, and this
 * plugin is used to warn the user when they try to import a built-in module,
 * when no fallback is configured.
 */
class SnapsBuiltInResolver {
    /**
     * The built-in modules that have been imported, but not resolved.
     */
    unresolvedModules = new Set();
    /**
     * The name of the resolver hook to tap into.
     */
    #source = 'described-resolve';
    /**
     * The options for the plugin.
     */
    options;
    constructor(options = {
        ignore: [],
    }) {
        this.options = options;
    }
    /**
     * Apply the plugin to the Webpack resolver.
     *
     * @param resolver - The Webpack resolver.
     */
    apply(resolver) {
        resolver
            .getHook(this.#source)
            .tapAsync(this.constructor.name, ({ module: isModule, request }, _, callback) => {
            if (!isModule || !request) {
                return callback();
            }
            const baseRequest = request.split('/')[0];
            if ((0, module_1.isBuiltin)(baseRequest) &&
                !this.options.ignore?.includes(baseRequest)) {
                const fallback = resolver.options.fallback.find(({ name }) => name === baseRequest);
                if (fallback && !fallback.alias) {
                    this.unresolvedModules.add(baseRequest);
                }
            }
            return callback();
        });
    }
}
exports.SnapsBuiltInResolver = SnapsBuiltInResolver;
/**
 * A plugin that logs a message when:
 *
 * - A built-in module is not resolved. The MetaMask Snaps CLI does not support
 * built-in modules by default, and this plugin is used to warn the user when
 * they try to import a built-in module, when no fallback is configured.
 * - A snap uses the `Buffer` global. The MetaMask Snaps CLI does not support
 * the `Buffer` global by default, and this plugin is used to warn the user when
 * they try to use the `Buffer` global.
 *
 * We use both a resolver and a plugin, because the resolver is used to detect
 * when a built-in module is imported, and the plugin is used to log a single
 * message when the compilation is complete. We can't do everything in a single
 * plugin, because the resolver doesn't have access to the compilation, and the
 * plugin doesn't have access to the resolver.
 */
class SnapsBundleWarningsPlugin {
    /**
     * The options for the plugin.
     */
    options;
    constructor(options = {
        buffer: true,
        builtIns: true,
    }) {
        this.options = options;
    }
    /**
     * Apply the plugin to the Webpack compiler.
     *
     * @param compiler - The Webpack compiler.
     */
    apply(compiler) {
        if (this.options.builtIns) {
            this.#checkBuiltIns(compiler);
        }
        if (this.options.buffer) {
            this.#checkBuffer(compiler);
        }
    }
    /**
     * Check if a built-in module is used, but not provided by Webpack's
     * `fallback` configuration.
     *
     * @param compiler - The Webpack compiler.
     */
    #checkBuiltIns(compiler) {
        compiler.hooks.afterCompile.tap(this.constructor.name, (compilation) => {
            if (!this.options.builtInResolver) {
                return;
            }
            const { unresolvedModules } = this.options.builtInResolver;
            if (unresolvedModules.size === 0) {
                return;
            }
            const formattedModules = new Array(...unresolvedModules)
                .map((name) => `• ${name}`)
                .join('\n');
            const webpackError = new webpack_1.WebpackError(`The snap attempted to use one or more Node.js builtins, but no browser fallback has been provided. The MetaMask Snaps CLI does not support Node.js builtins by default. If you want to use this module, you must set ${(0, chalk_1.bold)('`polyfills`')} to ${(0, chalk_1.bold)('`true`')} or an object with the builtins to polyfill as the key and ${(0, chalk_1.bold)('`true`')} as the value. To disable this warning, set ${(0, chalk_1.bold)('`stats.builtIns`')} to ${(0, chalk_1.bold)('`false`')} in your snap config file, or add the module to the ${(0, chalk_1.bold)('`stats.builtIns.ignore`')} array.`);
            webpackError.details = formattedModules;
            compilation.warnings.push(webpackError);
        });
    }
    /**
     * Check if the given instance is a `ProvidePlugin`. This is not guaranteed to
     * be accurate, but it's good enough for our purposes. If we were to use
     * `instanceof` instead, it might not work if multiple versions of Webpack are
     * installed.
     *
     * @param instance - The instance to check.
     * @returns Whether the instance is a `ProvidePlugin`, i.e., whether it's an
     * object with the name `ProvidePlugin` and a `definitions` property.
     */
    #isProvidePlugin(instance) {
        return ((0, utils_1.isObject)(instance) &&
            instance.constructor.name === 'ProvidePlugin' &&
            (0, utils_1.hasProperty)(instance, 'definitions'));
    }
    /**
     * Check if the `Buffer` global is used, but not provided by Webpack's
     * `DefinePlugin`.
     *
     * @param compiler - The Webpack compiler.
     */
    #checkBuffer(compiler) {
        const plugin = compiler.options.plugins?.find((instance) => this.#isProvidePlugin(instance));
        // If the `ProvidePlugin` is configured to provide `Buffer`, then we don't
        // need to warn the user.
        if (plugin) {
            const { definitions } = plugin;
            if (definitions.Buffer) {
                return;
            }
        }
        compiler.hooks.compilation.tap(this.constructor.name, (compilation) => {
            compilation.hooks.afterProcessAssets.tap(this.constructor.name, (assets) => {
                // Check if assets use `Buffer`.
                const bufferAssets = Object.entries(assets)
                    .filter(([name]) => name.endsWith('.js'))
                    .filter(([, asset]) => asset.source().includes('Buffer'));
                if (bufferAssets.length === 0) {
                    return;
                }
                compilation.warnings.push(new webpack_1.WebpackError(`The snap attempted to use the Node.js Buffer global, which is not supported in the MetaMask Snaps CLI by default. To use the Buffer global, you must polyfill Buffer by setting ${(0, chalk_1.bold)('`buffer`')} to ${(0, chalk_1.bold)('`true`')} in the ${(0, chalk_1.bold)('`polyfills`')} config object in your snap config. To disable this warning, set ${(0, chalk_1.bold)('`stats.buffer`')} to ${(0, chalk_1.bold)('`false`')} in your snap config file.`));
            });
        });
    }
}
exports.SnapsBundleWarningsPlugin = SnapsBundleWarningsPlugin;
//# sourceMappingURL=plugins.cjs.map