function $importDefault(module) {
    if (module?.__esModule) {
        return module.default;
    }
    return module;
}
import { NpmSnapFileNames, readJsonFile } from "@metamask/snaps-utils/node";
import $express, { static as expressStatic } from "express/index.js";
const express = $importDefault($express);
import { join, relative, resolve as resolvePath, sep, posix } from "path";
/**
 * Get the relative path from one path to another.
 *
 * Note: This is a modified version of `path.relative` that uses Posix
 * separators for URL-compatibility.
 *
 * @param from - The path to start from.
 * @param to - The path to end at.
 * @returns The relative path.
 */
function getRelativePath(from, to) {
    return relative(from, to).split(sep).join(posix.sep);
}
/**
 * Get the allowed paths for the static server. This includes the output file,
 * the manifest file, and any auxiliary/localization files.
 *
 * @param config - The config object.
 * @param manifest - The Snap manifest object.
 * @returns An array of allowed paths.
 */
export function getAllowedPaths(config, manifest) {
    const auxiliaryFiles = manifest.source.files?.map((file) => getRelativePath(config.server.root, resolvePath(config.server.root, file))) ?? [];
    const localizationFiles = manifest.source.locales?.map((localization) => getRelativePath(config.server.root, resolvePath(config.server.root, localization))) ?? [];
    const otherFiles = manifest.source.location.npm.iconPath
        ? [
            getRelativePath(config.server.root, resolvePath(config.server.root, manifest.source.location.npm.iconPath)),
        ]
        : [];
    return [
        getRelativePath(config.server.root, resolvePath(config.server.root, config.output.path, config.output.filename)),
        getRelativePath(config.server.root, resolvePath(config.server.root, NpmSnapFileNames.Manifest)),
        ...auxiliaryFiles,
        ...localizationFiles,
        ...otherFiles,
    ];
}
/**
 * Get whether the request path is allowed. This is used to check if the request
 * path is in the list of allowed paths for the static server.
 *
 * @param request - The request object.
 * @param config - The config object.
 * @returns A promise that resolves to `true` if the path is allowed, or
 * `false` if it is not.
 */
async function isAllowedPath(request, config) {
    const manifestPath = join(config.server.root, NpmSnapFileNames.Manifest);
    const { result } = await readJsonFile(manifestPath);
    const allowedPaths = getAllowedPaths(config, result);
    const path = request.path.slice(1);
    return allowedPaths.some((allowedPath) => path === allowedPath);
}
/**
 * Get a static server for development purposes.
 *
 * Note: We're intentionally not using `webpack-dev-server` here because it
 * adds a lot of extra stuff to the output that we don't need, and it's
 * difficult to customize.
 *
 * @param config - The config object.
 * @param middleware - An array of middleware functions to run before serving
 * the static files.
 * @returns An object with a `listen` method that returns a promise that
 * resolves when the server is listening.
 */
export function getServer(config, middleware = []) {
    const app = express();
    // Run "middleware" functions before serving the static files.
    middleware.forEach((fn) => fn(app));
    // Check for allowed paths in the request URL.
    app.use((request, response, next) => {
        isAllowedPath(request, config)
            .then((allowed) => {
            if (allowed) {
                // eslint-disable-next-line promise/no-callback-in-promise
                next();
                return;
            }
            response.status(404);
            response.end();
        })
            // eslint-disable-next-line promise/no-callback-in-promise
            .catch(next);
    });
    // Serve the static files.
    app.use(expressStatic(config.server.root, {
        dotfiles: 'deny',
        setHeaders: (res) => {
            res.setHeader('Cache-Control', 'no-cache');
            res.setHeader('Access-Control-Allow-Origin', '*');
        },
    }));
    /**
     * Start the server on the port specified in the config.
     *
     * @param port - The port to listen on.
     * @returns A promise that resolves when the server is listening. The promise
     * resolves to an object with the port and the server instance. Note that if
     * the `config.server.port` is `0`, the OS will choose a random port for us,
     * so we need to get the port from the server after it starts.
     */
    const listen = async (port = config.server.port) => {
        return new Promise((resolve, reject) => {
            // eslint-disable-next-line consistent-return
            const server = app.listen(port, (error) => {
                if (error) {
                    return reject(error);
                }
                const close = async () => {
                    await new Promise((resolveClose, rejectClose) => {
                        server.close((closeError) => {
                            if (closeError) {
                                return rejectClose(closeError);
                            }
                            return resolveClose();
                        });
                    });
                };
                const address = server.address();
                resolve({ port: address.port, server, close });
            });
        });
    };
    return { listen };
}
//# sourceMappingURL=server.mjs.map